/*
#pragma pack(2)
#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
*/
#include <Pilot.h>
#include <MemoryPrv.h>

#include "eCurrentDA.h"

#define CreatorID	'eCRT'
#define PrefID	0
#define PerfVersion	0
#define IntervalTicks	((sysTicksPerSecond)/10)
#define RomVersion20	0x02000000
#define	FullVoltage		300
#define DynamicHeapID	0

typedef struct {
	ULong timeSeconds;
	DateFormatType dateFormat;
	TimeFormatType timeFormat;
	Byte batteryPercent;
} DAGlobalType, *DAGlobalPtr;

typedef struct
{
	Boolean bSimpleForm;
} AppPrefType, *AppPrefPtr;

static void getObjectBoundsByID(Word, RectanglePtr);
static void drawStrToGadget(Word, CharPtr, FontID);
static Boolean displayDynamicInfo(DAGlobalPtr);
static Boolean displayDateAndTime(DAGlobalPtr);
static Boolean displayBattery(DAGlobalPtr);
static void displayStaticInfo(void);
static void displayMemory(void);
static void displayHeap(void);
static void StartApplication(void);
static void EventLoop(DAGlobalPtr);
static void StopApplication(void);
static Boolean DAHandleEvent(EventPtr, DAGlobalPtr);
void DAMain(void);

/* This code must be here. (first of code section)*/
void start()
{
	DAMain();
}

void DAMain(void)
{
	FormPtr pForm;
	DAGlobalType gStatus;

	StartApplication();
	
	gStatus.timeSeconds = 0;
	gStatus.dateFormat = (DateFormatType)PrefGetPreference(prefLongDateFormat);
	gStatus.timeFormat = (TimeFormatType)PrefGetPreference(prefTimeFormat);
	gStatus.batteryPercent = 0;

	// Activate Main Form
	pForm = FrmInitForm(frmMain);

	FrmSetActiveForm(pForm);
	FrmDrawForm(pForm);
	
	displayDynamicInfo(&gStatus);
	displayStaticInfo();

	// goto event handler
	EventLoop(&gStatus);

	FrmEraseForm(pForm);
	FrmDeleteForm(pForm);

	StopApplication();
}

static void StartApplication(void)
{
	DWord feature = 0;

	// Check duplicate invocation of eCurrentDA
	if (FtrGet(CreatorID, 0, &feature) == 0) {
		// Already eCurrentDA is invoked -- duplicate invocation
		return;
	} else {
		// To avoid duplicate invocation, set feature
		FtrSet(CreatorID, 0, feature);
	}
}

/* Event Loop */
static void EventLoop(DAGlobalPtr pStatus)
{
	EventType event;
	Boolean done = false;
	
	do {
		EvtGetEvent(&event, IntervalTicks);
		if (SysHandleEvent(&event))
			continue;

		done = DAHandleEvent(&event, pStatus);
	} while (!done);
}

static void StopApplication(void)
{
	DWord feature = 0;
	
	// Release feature
	if (FtrGet(CreatorID, 0, &feature) == 0) {
		FtrUnregister(CreatorID, 0);
	}
}

/* Event Handler */
static Boolean DAHandleEvent(EventPtr event, DAGlobalPtr pStatus)
{
	Boolean done = false;
	Boolean handled = false;

	switch (event->eType) {
		case appStopEvent:
			// Send appStopEvent to original application
			EvtAddEventToQueue(event);
			return true;
			
		case keyDownEvent:
			if (ChrIsHardKey(event->data.keyDown.chr)) {
				// maybe 'HotSync!' button pressed ?
				EvtAddEventToQueue(event);
				return true;
			}
			break;

		case ctlSelectEvent:
			if (event->data.ctlSelect.controlID == btnOK)
				return true;
			else
				break;

		case frmUpdateEvent:
			displayStaticInfo();
		case nilEvent:
			handled = displayDynamicInfo(pStatus);
   			break;

		default:
			break;
	}

	if (!handled)
		FrmHandleEvent(FrmGetActiveForm(), event);

	return done;
}

static void displayMemory(void)
{
	ULong storageHeapSize, freeBytes;
	ULong freeHeap, maxChunk;
	UInt numHeap, heapID;
//	UInt heapFlags, i;
	RectangleType rcMemory;
	Char freeStorage[7];

	storageHeapSize = freeBytes = 0;
	numHeap = MemNumRAMHeaps(0);

	for (heapID=1; heapID<numHeap; heapID++) {
		storageHeapSize += MemHeapSize(heapID);
		MemHeapFreeBytes(heapID, &freeHeap, &maxChunk);
		freeBytes += freeHeap;
	}

	getObjectBoundsByID(gdtMemory, &rcMemory);
	WinDrawRectangleFrame(simpleFrame, &rcMemory);

	rcMemory.extent.x = rcMemory.extent.x*(storageHeapSize-freeBytes)/storageHeapSize;
	WinDrawRectangle(&rcMemory, 0);

	StrPrintF(freeStorage, "%ldK", freeBytes/1024);
	drawStrToGadget(gdtFreeMem, freeStorage, boldFont);
}

static void displayHeap(void)
{
	ULong dynamicHeapSize, freeBytes, maxChunk;
	RectangleType rcHeap;
	Char freeDynamic[5];
	
	dynamicHeapSize = MemHeapSize(DynamicHeapID);
	MemHeapFreeBytes(DynamicHeapID, &freeBytes, &maxChunk);
	
	getObjectBoundsByID(gdtHeap, &rcHeap);
	WinDrawRectangleFrame(simpleFrame, &rcHeap);

	rcHeap.extent.x = rcHeap.extent.x*(dynamicHeapSize-freeBytes)/dynamicHeapSize;
	WinDrawRectangle(&rcHeap, 0);

	StrPrintF(freeDynamic, "%ldK", freeBytes/1024);
	drawStrToGadget(gdtFreeHeap, freeDynamic, boldFont);
}

static void displayStaticInfo(void)
{
	displayMemory();
	displayHeap();
}

static Boolean displayDynamicInfo(DAGlobalPtr pStatus)
{
	if (displayDateAndTime(pStatus))
		return displayBattery(pStatus);
	else
		return false;
}
static Boolean displayDateAndTime(DAGlobalPtr pStatus)
{
	ULong curSeconds;
	DateTimeType dateTime;
	Char date[longDateStrLength], time[timeStringLength+10], sec[4];
	CharPtr week, weekDay[7] = {"Sun.", "Mon.", "Tue.", "Wed.", "Thu.", "Fri.", "Sat."};

	curSeconds = TimGetSeconds();

	if (curSeconds != pStatus->timeSeconds) {
		pStatus->timeSeconds = curSeconds;

		TimSecondsToDateTime(curSeconds, &dateTime);
		DateToAscii(dateTime.month, dateTime.day, dateTime.year, pStatus->dateFormat, date);

		week = weekDay[dateTime.weekDay];
	
		TimeToAscii(dateTime.hour, dateTime.minute, tfColon24h, time);
		
		if (dateTime.second < 10)
			StrCat(time, ":0");
		else
			StrCat(time, ":");
			
		StrIToA(sec, dateTime.second);
		StrCat(time, sec);

		drawStrToGadget(gdtDate, date, stdFont);
		drawStrToGadget(gdtWeek, week, stdFont);
		drawStrToGadget(gdtTime, time, boldFont);

		return true;
	}

	return false;
}

static Boolean displayBattery(DAGlobalPtr pStatus)
{
	RectangleType rcBattery;
	RectangleType rcBig, rcSmall;
	Word smallWidth, smallHeight;
	Byte remainingPercent;
	UInt currentVoltage, warnThreshold;
	Char voltage[7];

	currentVoltage = SysBatteryInfo(false, &warnThreshold, 0, 0, 0, 0);

	if (warnThreshold < FullVoltage)
	{
		if (currentVoltage > warnThreshold)
			remainingPercent = 100*(currentVoltage-warnThreshold)/(FullVoltage-warnThreshold);
		else
			remainingPercent = 0;
	}
	else
		remainingPercent = 100;

	if (remainingPercent > 100) remainingPercent = 100;

	// If percent is same as old, do nothing
	if (remainingPercent == pStatus->batteryPercent) 
		return false;
	else
		pStatus->batteryPercent = remainingPercent;

	// Save new battery percent
	pStatus->batteryPercent = remainingPercent;
	
	getObjectBoundsByID(gdtBattery, &rcBattery);

	smallWidth = rcBattery.extent.x / 20;
	smallHeight = rcBattery.extent.y / 4;

	rcBig = rcBattery;
	rcBig.extent.x -= smallWidth;

	rcSmall.topLeft.x = rcBig.topLeft.x + rcBig.extent.x;
	rcSmall.topLeft.y = rcBig.topLeft.y + smallHeight;
	rcSmall.extent.x = smallWidth;
	rcSmall.extent.y = rcBig.extent.y - smallHeight*2;

	rcBig.extent.x--;

	// Draw the frame of the battery gadget.
	WinDrawRectangleFrame(simpleFrame, &rcBig);
	WinDrawRectangleFrame(simpleFrame, &rcSmall);
	WinEraseLine(rcSmall.topLeft.x-1, rcSmall.topLeft.y, 
		rcSmall.topLeft.x-1, rcSmall.topLeft.y+rcSmall.extent.y-1);

	// Draw the power indicator.
	rcBig.topLeft.x++;
	rcBig.topLeft.y++;
	rcBig.extent.x = (remainingPercent * (rcBig.extent.x-2) + 50) / 100;
	rcBig.extent.y -= 2;
	WinDrawRectangle(&rcBig, 0);
	
	rcBig.topLeft.x += rcBig.extent.x;
	rcBig.extent.x = rcSmall.topLeft.x - rcBig.topLeft.x - 2;
	WinEraseRectangle(&rcBig, 0);
	
	StrIToA(voltage, currentVoltage);
	voltage[3] = voltage[2];
	voltage[2] = voltage[1];
	voltage[1] = '.';
	voltage[4] = 'V';
	voltage[5] = 0;

	drawStrToGadget(gdtVoltage, voltage, boldFont);
	
	return true;
}

static void drawStrToGadget(Word gadgetID, CharPtr str, FontID fnt)
{
	FontID prevFont;
	RectangleType rect;

	// Save the current font.
	prevFont = FntSetFont(fnt);

	getObjectBoundsByID(gadgetID, &rect);
	WinEraseRectangle(&rect, 0);
	WinDrawChars(str, StrLen(str), rect.topLeft.x, rect.topLeft.y);

	// Restore the font.
	FntSetFont(prevFont);
}

static void getObjectBoundsByID(Word objID, RectanglePtr rcP)
{
	FormPtr frm = FrmGetActiveForm();
	FrmGetObjectBounds(frm, FrmGetObjectIndex(frm, objID), rcP);
}